@@ -1,5 +1,7 @@
 #include <hyperg.h>
 
+
+// 22/02/2012: corrected Laplace approximation formula
 // compute the log of the psi function,
 // either by calling the hyp2f1 function or by a Laplace approximation
 double logPsi(double b, double c, int n, int p, double R2){
@@ -25,29 +27,27 @@ double logPsi(double b, double c, int n, int p, double R2){
     	// there were numerical problems with hyp2f1, so we do a Laplace approximation:
     	
     	// components of quadratic equation in exp(tau) are
-    	double logAlpha = log(2 * b - pmodel - c) + log1p(-R2);
-    	double alpha = exp(logAlpha);
+    	double alpha = (2 * b - pmodel - c) * (1 - R2);
     	
     	double beta = 4 * b - pmodel - c + R2 * (n - 1 - 2 * b);
     	
     	double gamma = 2 * b;
     	
+    	// exp(hat(tau)) is the mode of g:
+    	double gMode = (- beta - sqrt(pow(beta, 2.0) - 4 * alpha * gamma)) / (2 * alpha);
+
     	// hat(tau)
-    	double tauMode = log(- beta + sqrt(pow(beta, 2.0) - 4 * alpha * gamma)) 
-    					- M_LN2 - logAlpha;
-    	
-       	// exp(hat(tau)) is the mode of g:
-    	double gMode = exp(tauMode);
-    	
+    	double tauMode = log(gMode);
+
     	// the log integrand evaluated at the mode is
     	double logIntegrandAtTauMode = b * tauMode 
-    								+ (n - 1 - pmodel - c) / 2.0 * log1p(gMode)
-    								- (n - 1) / 2.0 * log1p((1 - R2) * gMode);
+    				       + (n - 1 - pmodel - c) / 2.0 * log1p(gMode)
+    				       - (n - 1) / 2.0 * log1p((1 - R2) * gMode);
     	
     	// the log variance is
     	double logSigma2 = 
-    		- M_LN2
-    		+ tauMode
+    		M_LN2
+    		- tauMode
     		- logspace_sub(
     				log(static_cast<double>(n - 1)) + log1p(-R2) - 2 * log1p((1 - R2) * gMode),
     				log(n - 1 - pmodel - c) - 2 * log1p(gMode) 
@@ -72,7 +72,7 @@ double logBF_hyperg(double R2, int n, int p, double alpha)
      hyperg prior for regression coefficients; alpha > 2 
      log marginal for null model is 0 */
 
-	double ret;
+    double ret;
 	
     if (p == 1) 
     	ret = 0.0;
